#!/bin/sh
#
# js2flash-testrunner.sh: automated test runner for js2flash-auto
# 
# js2flash-auto, automatic ExternalInterface test code for GNU Flash Player
# Copyright (C) 2015 Nutchanon Wetchasit
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
# 
# Usage:
#     ./js2flash-testrunner.sh
#
# Exit codes:
#     0         if all test passed
#     non-zero  if there are failed test, or tester encountered an error
#
# Note:
#     This test requires a writable test directory, and a filesystem
#     that supports named pipe.
#

# Filenames and constants
LOGFILE=output.log
PIPE2CONTAINER=pipe2container
PIPE2PLAYER=pipe2player
READTIMEOUT=5

# Test counts
TESTED=0
FAILED=0
PASSED=0

# check_equals($op1, $op2, $msg)
# Equality checker and counter
check_equals() {
	if [ "$1" = "$2" ]
	then
		echo "PASSED: $3"
		PASSED=`expr "$PASSED" + 1`
	else
		echo "FAILED: $3 (\"$1\" != \"$2\")"
		FAILED=`expr "$FAILED" + 1`
	fi
	TESTED=`expr "$TESTED" + 1`
}

# check_error($bool, $msg)
# Assert $bool is 0; if not, flag error in the test, and exit
check_error() {
	if [ "$1" -ne 0 ]
	then
		echo "ERROR: $2" >&2
		exit 1
	fi
}

# read_timeout($varname, $timeout)
# Read one line from standard input, with a specified timeout (in seconds)
read_timeout() {
	trap 'trap - USR1; return 142' USR1
	(sleep "$2" && kill -USR1 "$$" > /dev/null 2>&1) &
	TIMEOUTPID=$!
	read "$1"
	READERROR=$?
	kill "$TIMEOUTPID" > /dev/null 2>&1
	trap - USR1
	return $READERROR
}

# Create required named pipes
if [ \! -p "$PIPE2CONTAINER" ]
then
	mkfifo "$PIPE2CONTAINER"
	check_error "$?" "Failed to create a named pipe: $PIPE2CONTAINER"
fi
if [ \! -p "$PIPE2PLAYER" ]
then
	mkfifo "$PIPE2PLAYER"
	check_error "$?" "Failed to create a named pipe: $PIPE2PLAYER"
fi

# Open player-to-host pipe
exec 3<> "$PIPE2CONTAINER"
check_error $? "Failed to open a named pipe: $PIPE2CONTAINER"

# Open host-to-player pipe
exec 4<> "$PIPE2PLAYER"
check_error $? "Failed to open a named pipe: $PIPE2CONTAINER"

# Start player
dump-gnash -r 2 -t 5 -v -F 3:4 js2flash-auto.swf > $LOGFILE 2>&1 &
GNASHPID=$!

# Read for callback registration statement
read_timeout LINE $READTIMEOUT <&3
check_equals \
	"$LINE" \
	'<invoke name="addMethod" returntype="xml"><arguments><string>script_call</string></arguments></invoke>' \
	"Gnash should properly register an ExternalInterface callback"

# Call the callback
echo '<invoke name="script_call" returntype="xml"><arguments><string>Hello</string><string>World</string></arguments></invoke>' >&4

# Read for callback return value statement
read_timeout LINE $READTIMEOUT <&3
check_equals "$LINE" '<string>Too</string>' "Gnash should return a correct value from ExternalInterface callback"

# Wait for Gnash to exit
wait $GNASHPID
check_equals "$?" "0" "Gnash should terminate successfully"

# Show result from Flash's side
exec 5< "$LOGFILE"
grep "TRACE:" "$LOGFILE" | sed 's/^.*TRACE: //g'
exec 5<&-

# Include total number of tests from player side
exec 5< "$LOGFILE"
PLAYERPASSED=`egrep "TRACE: PASSED:" <&5 | wc -l`
exec 5<&-
exec 5< "$LOGFILE"
PLAYERFAILED=`egrep "TRACE: FAILED:" <&5 | wc -l`
exec 5<&-
PASSED=`expr "$PASSED" + "$PLAYERPASSED"`
FAILED=`expr "$FAILED" + "$PLAYERFAILED"`
TESTED=`expr "$TESTED" + "$PLAYERPASSED" + "$PLAYERFAILED"`

# Check total number of tests
check_equals "$TESTED" "7" "There should be 7 tests run"

# Exit with code 0 if no failure, otherwise 1
if [ "$FAILED" -eq "0" ]
then
	exit 0
else
	exit 1
fi
